instruments =['000021.SZA','000034.SZA','000066.SZA','000158.SZA','000555.SZA','000606.SZA','000662.SZA','000938.SZA','000948.SZA','000606.SZA','300157.SZA','300164.SZA','300191.SZA','600121.SHA','600123.SHA','600157.SHA','600188.SHA','600193.SHA','600295.SHA','600339.SHA','300610.SZA','300637.SZA','300641.SZA','300655.SZA','300665.SZA','300690.SZA','300699.SZA','300716.SZA','300717.SZA','300721.SZA','000060.SZA','000426.SZA','000511.SZA','000603.SZA','000612.SZA','000630.SZA','000633.SZA','000657.SZA','000688.SZA','000693.SZA','300592.SZA','300621.SZA','300635.SZA','300649.SZA','300668.SZA','600039.SHA','600068.SHA','600133.SHA','600170.SHA','600209.SHA','603218.SHA','603320.SHA','603333.SHA','603396.SHA','603416.SHA','603488.SHA','603507.SHA','603577.SHA','603606.SHA','603488.SHA','300403.SZA','300475.SZA','600060.SHA','600336.SHA','600619.SHA','600690.SHA','600839.SHA','600854.SHA','600870.SHA','600983.SHA','600867.SHA','600896.SHA','600976.SHA','600993.SHA','600998.SHA','601607.SHA','603079.SHA','603108.SHA','603127.SHA','603139.SHA','002351.SZA','002369.SZA','002371.SZA','002388.SZA','002384.SZA','002402.SZA','002414.SZA','002463.SZA','002475.SZA','002484.SZA','002711.SZA','002769.SZA','002800.SZA','002889.SZA','300013.SZA','300240.SZA','300350.SZA','600004.SHA','600009.SHA','600026.SHA','601128.SHA','601166.SHA','601169.SHA','601229.SHA','601288.SHA','601398.SHA','601328.SHA','601818.SHA','601939.SHA','601988.SHA','600804.SHA','603042.SHA','603083.SHA','603118.SHA','603322.SHA','603421.SHA','603559.SHA','603602.SHA','603703.SHA','603803.SHA']

start_date = '20190103'
end_date = '20210122'

import json
import talib
import numpy as np


def initialize(context):
    # MACD 金叉MA 多头 指标设置DIF 短线=12, DIF 长线=26, DEA=9
    context.short_period = 12
    context.long_period = 26
    context.dea = 9

    context.trading_day_count = 0

    # MA 多头 MA 短线=5，MA 长线=20
    context.ma_short = 5
    context.ma_long = 20

    # 资金
    context.cash = 10000

    # 交易费用
    context.set_commission(PerOrder(buy_cost=0.0003, sell_cost=0.0003, min_cost=5))


def is_all_nan(data):
    for datum in data:
        if str(datum) != 'nan':
            return False
    return True


# 计算MACD
def macd(context, data):
    stock_set = set()
    d = data.history(data.keys(), 'price', bar_count=34, frequency='1d')
    for key in data.keys():
        s = np.asarray(d[key])
        if not is_all_nan(s):
            macd, macdsignal, macdhist = talib.MACD(s, context.short_period, context.long_period, context.dea)
            if macdhist[-1] > 0:
                stock_set.add(key.symbol)
    return stock_set


# 计算MA
def ma(context, data):
    stock_set = set()
    d_5 = data.history(data.keys(), 'price', bar_count=5, frequency='1d')
    d_20 = data.history(data.keys(), 'price', bar_count=20, frequency='1d')

    for key in data.keys():
        s_5 = np.asarray(d_5[key])
        s_20 = np.asarray(d_20[key])
        if not is_all_nan(s_5) and not is_all_nan(s_20):
            ma5 = talib.MA(s_5, timeperiod=5, matype=0)
            ma20 = talib.MA(s_20, timeperiod=20, matype=0)
            if ma5[-1] > ma20[-1]:
                stock_set.add(key.symbol)

    return stock_set


def handle_data(context, data):
    # 每20日交易一次
    if context.trading_day_count % 20 == 0:

        set_macd = macd(context, data)
        set_ma = ma(context, data)
        candidate_list = list(set_macd & set_ma)
        max_stock_count = 10
        if len(candidate_list) > max_stock_count:
            candidate_list = candidate_list[0:10]

        # 全部卖出
        for equity in context.portfolio.positions:
            position = context.portfolio.positions[equity]
            sid = position.sid
            if data.can_trade(sid):
                context.order_target_percent(sid, 0)

        cash = context.portfolio.cash

        # 买入
        for stock in candidate_list:
            sid = context.symbol(stock)
            if data.can_trade(sid):
                price = data.current(sid, 'price')
                context.order(sid, int(cash / price / 100 / len(candidate_list)) * 100)

                # 交易日+1
        context.trading_day_count = context.trading_day_count + 1


m=M.trade.v2(
    instruments=instruments,
    start_date=start_date,
    end_date=end_date,
    initialize=initialize,
    handle_data=handle_data,
    order_price_field_buy='open', # 以开盘价买入
    order_price_field_sell='open', # 以开盘价卖出
    capital_base=100000, # 本金
    m_deps=np.random.randn()
    )