instruments =['000021.SZA','000034.SZA','000066.SZA','000158.SZA','000555.SZA','000606.SZA','000662.SZA','000938.SZA','000948.SZA','000606.SZA','300157.SZA','300164.SZA','300191.SZA','600121.SHA','600123.SHA','600157.SHA','600188.SHA','600193.SHA','600295.SHA','600339.SHA','300610.SZA','300637.SZA','300641.SZA','300655.SZA','300665.SZA','300690.SZA','300699.SZA','300716.SZA','300717.SZA','300721.SZA','000060.SZA','000426.SZA','000511.SZA','000603.SZA','000612.SZA','000630.SZA','000633.SZA','000657.SZA','000688.SZA','000693.SZA','300592.SZA','300621.SZA','300635.SZA','300649.SZA','300668.SZA','600039.SHA','600068.SHA','600133.SHA','600170.SHA','600209.SHA','603218.SHA','603320.SHA','603333.SHA','603396.SHA','603416.SHA','603488.SHA','603507.SHA','603577.SHA','603606.SHA','603488.SHA','300403.SZA','300475.SZA','600060.SHA','600336.SHA','600619.SHA','600690.SHA','600839.SHA','600854.SHA','600870.SHA','600983.SHA','600867.SHA','600896.SHA','600976.SHA','600993.SHA','600998.SHA','601607.SHA','603079.SHA','603108.SHA','603127.SHA','603139.SHA','002351.SZA','002369.SZA','002371.SZA','002388.SZA','002384.SZA','002402.SZA','002414.SZA','002463.SZA','002475.SZA','002484.SZA','002711.SZA','002769.SZA','002800.SZA','002889.SZA','300013.SZA','300240.SZA','300350.SZA','600004.SHA','600009.SHA','600026.SHA','601128.SHA','601166.SHA','601169.SHA','601229.SHA','601288.SHA','601398.SHA','601328.SHA','601818.SHA','601939.SHA','601988.SHA','600804.SHA','603042.SHA','603083.SHA','603118.SHA','603322.SHA','603421.SHA','603559.SHA','603602.SHA','603703.SHA','603803.SHA']

start_date = '20190103'
end_date = '20210122'

import json
import talib
import numpy as np
import cvxopt as opt
from cvxopt import blas, solvers

solvers.options['show_progress'] = False

def initialize(context):

    context.set_commission(PerOrder(buy_cost=0.003, sell_cost=0.003, min_cost=5))


def optimal_portfolio(returns):
    returns = np.asmatrix(returns)
    n = len(returns)

    # 转化为cvxopt matrices
    # Σ: 这是收益的协方差矩阵，代表着股票之间的关系
    S = opt.matrix(np.cov(returns))
    # μ = (μ1, ..., μn)：这是通过历史数据计算出来的每只股票的期望收益。
    pbar = opt.matrix(np.mean(returns, axis=1))

    # 约束条件
    G = -opt.matrix(np.eye(n))
    h = opt.matrix(0.0, (n, 1))
    A = opt.matrix(1.0, (1, n))
    b = opt.matrix(1.0)

    # 使用凸优化计算有效前沿
    # 计算最优组合
    wt = solvers.qp(opt.matrix(0.1 * S), -pbar, G, h, A, b)['x']
    return np.asarray(wt)


def handle_data(context, data):
    if context.trading_day_index % 20 == 0:

        prices = data.history(data.keys(), fields='price', bar_count=6, frequency='1d').dropna(
            axis=1).pct_change().dropna(axis=0)

        if not prices.empty:
            weights = optimal_portfolio(prices.T)
            for stock, weight in zip(prices.columns, weights):
                if data.can_trade(stock):
                    context.order_target_percent(stock, weight[0])

m=M.trade.v2(
    instruments=instruments,
    start_date=start_date,
    end_date=end_date,
    initialize=initialize,
    handle_data=handle_data,
    order_price_field_buy='open', # 以开盘价买入
    order_price_field_sell='open', # 以开盘价卖出
    capital_base=100000, # 本金
    m_deps=np.random.randn()
    )