


start_date = '20190103'
end_date = '20210122'

import pandas as pd
import json
import talib
import numpy as np
import cvxopt as opt
from cvxopt import blas, solvers

solvers.options['show_progress'] = False

# 股票
df = pd.read_csv('stock_list.csv',header=None)
instruments = df[0].to_list()

# 每个行业下股票数量
industries = df[1].to_list()
industries_cnt = [industries.count(i) for i in np.unique(industries)]


def initialize(context):
    # 设置手续费
    context.set_commission(PerOrder(buy_cost=0.003, sell_cost=0.003, min_cost=5))

    context.rebalance_period = 20
    context.observation = 5
    context.max_weight_per_stock = 0.1
    context.max_weight_per_industry = 0.3


def optimal_portfolio(context, returns):
    returns = np.asmatrix(returns)
    n = len(returns)

    # 转化为cvxopt matrices
    S = opt.matrix(np.cov(returns))
    pbar = opt.matrix(np.mean(returns, axis=1))

    # 初始约束条件
    G1 = -opt.matrix(np.eye(n))
    h1 = opt.matrix(0.0, (n, 1))
    # 每只股票权重限制
    G2 = opt.matrix(np.eye(n))
    h2 = opt.matrix(context.max_weight_per_stock, (n, 1))
    # 行业限制
    G3 = np.zeros((len(industries_cnt), n))
    start = 0
    end = 0
    for i in range(len(industries_cnt)):
        end += industries_cnt[i]
        G3[i, start:end] = 1
        start = end
    G3 = opt.matrix(G3)
    h3 = opt.matrix(context.max_weight_per_industry, (len(industries_cnt), 1))
    G = opt.matrix(np.concatenate([G1, G2, G3]))
    h = opt.matrix(np.concatenate([h1, h2, h3]))
    A = opt.matrix(1.0, (1, n))
    b = opt.matrix(1.0)

    # 使用凸优化计算有效前沿
    # 计算最优组合
    wt = solvers.qp(opt.matrix(0.1 * S), -pbar, G, h, A, b)['x']
    return np.asarray(wt)


def handle_data(context, data):
    if context.trading_day_index % 20 == 0:

        prices = data.history(data.keys(), fields='price', bar_count=6, frequency='1d').dropna(
            axis=1).pct_change().dropna(axis=0)

        if not prices.empty:
            weights = optimal_portfolio(context, prices.T)
            for stock, weight in zip(prices.columns, weights):
                if data.can_trade(stock):
                    context.order_target_percent(stock, weight[0])

m=M.trade.v2(
    instruments=instruments,
    start_date=start_date,
    end_date=end_date,
    initialize=initialize,
    handle_data=handle_data,
    order_price_field_buy='open', # 以开盘价买入
    order_price_field_sell='open', # 以开盘价卖出
    capital_base=100000, # 本金
    m_deps=np.random.randn()
    )