
start_date = '20190103'
end_date = '20210122'

import pandas as pd
import json
import talib
import numpy as np
import cvxopt as opt
from cvxopt import blas, solvers

solvers.options['show_progress'] = False

# 股票
df = pd.read_csv('stock_list.csv',header=None)
instruments = df[0].to_list()

# 每个行业下股票数量
industries = df[1].to_list()
industries_cnt = [industries.count(i) for i in np.unique(industries)]


def initialize(context):
    # 设置手续费
    context.set_commission(PerOrder(buy_cost=0.003, sell_cost=0.003, min_cost=5))

    context.rebalance_period = 20
    context.observation = 5
    context.max_weight_per_stock = 0.1
    context.max_weight_per_industry = 0.3


def optimal_portfolio(context, returns):
    returns = np.asmatrix(returns)
    n = len(returns)

    # 转化为cvxopt matrices
    S = opt.matrix(np.cov(returns))
    pbar = opt.matrix(np.mean(returns, axis=1))

    # 初始约束条件
    G1 = -opt.matrix(np.eye(n))  # opt默认是求最大值，因此要求最小化问题，还得乘以一个负号
    h1 = opt.matrix(0.0, (n, 1))
    # 每只股票权重限制
    G2 = opt.matrix(np.eye(n))
    h2 = opt.matrix(context.max_weight_per_stock, (n, 1))
    # 行业限制
    G3 = np.zeros((len(industries_cnt), n))
    start = 0
    end = 0
    for i in range(len(industries_cnt)):
        end += industries_cnt[i]
        G3[i, start:end] = 1
        start = end
    G3 = opt.matrix(G3)
    h3 = opt.matrix(context.max_weight_per_industry, (len(industries_cnt), 1))
    G = opt.matrix(np.concatenate([G1, G2, G3]))
    h = opt.matrix(np.concatenate([h1, h2, h3]))
    A = opt.matrix(1.0, (1, n))
    b = opt.matrix(1.0)

    # 使用凸优化计算有效前沿
    # 计算最优组合
    wt = solvers.qp(opt.matrix(0.1 * S), -pbar, G, h, A, b)['x']
    return np.asarray(wt)


def is_all_nan(data):
    for datum in data:
        if str(datum) != 'nan':
            return False
    return True


def min_cap(context, data):
    market_cap_data = D.history_data(instruments, data.current_dt, data.current_dt,
                                     fields=['market_cap', 'amount', 'suspended'])
    # 根据是否停牌的字段确定每日选出来的股票
    daily_buy_stock = market_cap_data.groupby('date').apply(lambda df: df[(df['amount'] > 0)  # 需要有成交量
                                                                          & (df['suspended'] == False)  # 是否停牌
                                                                          ].sort_values('market_cap')[:30])  #
    # context.logger.info(daily_buy_stock['instrument'])
    return set(daily_buy_stock['instrument'])


# 计算MA
def ma(context, data):
    stock_set = set()
    d_5 = data.history(data.keys(), 'price', bar_count=5, frequency='1d')
    d_20 = data.history(data.keys(), 'price', bar_count=20, frequency='1d')

    for key in data.keys():
        s_5 = np.asarray(d_5[key])
        s_20 = np.asarray(d_20[key])
        if not is_all_nan(s_5) and not is_all_nan(s_20):
            ma5 = talib.MA(s_5, timeperiod=5, matype=0)
            ma20 = talib.MA(s_20, timeperiod=20, matype=0)
            if ma5[-1] > ma20[-1]:
                stock_set.add(key.symbol)

    return stock_set


def handle_data(context, data):
    if context.trading_day_index % 20 == 0:

        set_cap = min_cap(context, data)
        set_ma = ma(context, data)

        candidate_list = list(set_cap & set_ma)

        candidate = []

        for stock in candidate_list:
            candidate.append(context.symbol(stock))

        if len(candidate) > 1:
            prices = data.history(candidate, fields='price', bar_count=6, frequency='1d').dropna(
                axis=1).pct_change().dropna(axis=0)

            if not prices.empty:
                weights = optimal_portfolio(context, prices.T)
                for stock, weight in zip(prices.columns, weights):
                    if data.can_trade(stock):
                        context.order_target_percent(stock, weight[0])

m=M.trade.v2(
    instruments=instruments,
    start_date=start_date,
    end_date=end_date,
    initialize=initialize,
    handle_data=handle_data,
    order_price_field_buy='open', # 以开盘价买入
    order_price_field_sell='open', # 以开盘价卖出
    capital_base=100000, # 本金
    m_deps=np.random.randn()
    )